import logging
import psycopg2
import threading
from datetime import datetime
from config import Config

DATABASE_URI = Config.DATABASE_URI # Use the PostgreSQL URI directly
LOG_TABLE_NAME = 'agent_logs'

# Use a thread-local storage for database connections to avoid issues with
# different threads (like APScheduler) using the same connection.
local_storage = threading.local()

def get_db_connection_for_logging():
    """Gets a DB connection specific to the current thread for logging."""
    if not hasattr(local_storage, 'log_db_conn'):
        try:
            # Connect using the PostgreSQL URI from Config
            local_storage.log_db_conn = psycopg2.connect(DATABASE_URI)
        except psycopg2.Error as e:
            print(f"FATAL: Could not connect to PostgreSQL database for logging: {e}") # Use print as logging might not be working
            local_storage.log_db_conn = None
    return local_storage.log_db_conn

class DatabaseLogHandler(logging.Handler):
    """
    A logging handler that writes log records to a PostgreSQL database table.
    """
    def __init__(self):
        super().__init__()

    def emit(self, record):
        """
        Writes the log record to the database.
        """
        conn = get_db_connection_for_logging()
        if conn is None:
            print(f"ERROR: No database connection available for logging record: {record.getMessage()}")
            return # Cannot log if connection failed

        try:
            # Format the log record
            log_timestamp = datetime.fromtimestamp(record.created)
            agent_name = getattr(record, 'agent_name', record.name) # Use custom attribute if available
            level = record.levelname
            message = self.format(record) # Use formatter if one is set, otherwise just the message
            details = None
            if record.exc_info:
                details = logging.Formatter().formatException(record.exc_info)

            # Insert into database
            cursor = conn.cursor()
            cursor.execute(f"""
                INSERT INTO {LOG_TABLE_NAME} (timestamp, agent_name, level, message, details) 
                VALUES (%s, %s, %s, %s, %s)
            """, (log_timestamp, agent_name, level, message, details))
            conn.commit()

        except psycopg2.Error as e:
            # Use print for PostgreSQL errors occurring within the log handler itself
            print(f"Database logging error: {e}\nFailed Record: {record.getMessage()}")
        except Exception as e:
            print(f"Unexpected error in DatabaseLogHandler: {e}\nFailed Record: {record.getMessage()}")

    def close(self):
        """Closes the database connection for the current thread."""
        if hasattr(local_storage, 'log_db_conn') and local_storage.log_db_conn:
            local_storage.log_db_conn.close()
            del local_storage.log_db_conn
        super().close()


def setup_logging():
    """
    Configures the root logger to use the DatabaseLogHandler
    and optionally a StreamHandler for console output.
    """
    db_handler = DatabaseLogHandler()
    # Optional: Add a formatter for more detailed logs in the DB
    # formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # db_handler.setFormatter(formatter)

    # Configure console logging as well (useful for development/debugging)
    console_handler = logging.StreamHandler()
    console_formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    console_handler.setFormatter(console_formatter)

    # Get the root logger and add handlers
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.INFO) # Set the desired logging level (e.g., INFO, DEBUG)

    # Avoid adding handlers multiple times if setup_logging is called again
    if not any(isinstance(h, DatabaseLogHandler) for h in root_logger.handlers):
        root_logger.addHandler(db_handler)
    if not any(isinstance(h, logging.StreamHandler) for h in root_logger.handlers):
        root_logger.addHandler(console_handler)

    # Also configure the logger used within db_utils
    db_utils_logger = logging.getLogger('news_system.db_utils')
    db_utils_logger.setLevel(logging.INFO) # Or DEBUG if needed

    print("Logging configured with DatabaseLogHandler and StreamHandler.")
